/* Esta procedure  realiza o clculo das datas de incio e trmino das atividades
@C:\Web_511\Sql\End_Conv\Oracle\9060_GEPSTPO_CALCULA_DATAS.SQL
execute CALCULA_CRONOGRAMA('9999', 'FIS0000001', '001', '001')
*/

CREATE OR REPLACE PROCEDURE CALCULA_DATAS(
     P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
     P_CDPROJETO    IN PROJETO.CDPROJETO%TYPE,
     P_NRBASELINEPR IN CRONOPROJETO.NRBASELINEPR%TYPE,
     P_CDVERCRONPRJ IN CRONOPROJETO.CDVERCRONPRJ%TYPE,
     P_CDOPERADOR   IN OPERADOR.CDOPERADOR%TYPE,
     P_FILAPRIO     IN CRONOPROJETO.CDFILAPRIO%TYPE,
     P_DATAINIPRJ   IN CRONOPROJETO.DTINIITCRPRJ%TYPE)
AS
   V_DATAFINAL       DATE;
   V_DATAFINANT      DATE;
   V_DATAINI         DATE;
   V_DATAINCL        DATE;
   V_DATAATIV        DATE;
   V_DATANOVA        DATE;
   V_DTOCUPOPER      DATE;
   V_PRODDIA         FLOAT;
   V_QTTEMPPRODIA    FLOAT;
   V_QTHRATIV        FLOAT;
   V_QTHRPROD        FLOAT;
   V_QTOPUCOPER      FLOAT;
   V_DIAS            FLOAT;
   V_DECIMAL         FLOAT;
   V_DIASANT         FLOAT;
   V_DIASINI         FLOAT;
   V_PROJETO         VARCHAR2(10);
   V_PRECEDENCIA     VARCHAR2(3);
   V_FILA            VARCHAR2(3);
   V_CDVERSAO        VARCHAR2(3);
   V_IDSITATIVTAR    VARCHAR2(1);
   V_OPERADORPREC    OPERADOR.CDOPERADOR%TYPE;
   V_COMPLEXIDADE    FLOAT;   /*Valor da complexidade*/
   V_NRCPLXITATIV    FLOAT;   /*Nmero da complexidade da atividade*/
   V_IDATVTEMPALTCR  VARCHAR2(1);
   V_IDATIVINCLCRONO VARCHAR2(1);
BEGIN
   /* Atividades do operador que ainda no possuem data de inicio e tenha a prioridade menor do que a informada
      Desta forma o clculo ser at na prioridade informada quando possuir precedncia*/
   FOR C_TAREFAS IN (SELECT CDSEQCRONPRJ, NRSOLISERV, NRTAREFA, NRPLATIVTARE, CDFILAPRIO, CDPRECITCRPJ,
                            CDOPERADOR, QTDIAFOLGACRON, NRDIASITCRPJ
                       FROM CRONOPROJETO
                      WHERE CDFILIAL     = P_CDFILIAL
                        AND CDPROJETO    = P_CDPROJETO
                        AND NRBASELINEPR = P_NRBASELINEPR
                        AND CDVERCRONPRJ = P_CDVERCRONPRJ
                        AND CDOPERADOR   = P_CDOPERADOR
                        AND DTINIITCRPRJ IS NULL
                        AND CDFILAPRIO   <= P_FILAPRIO
                        AND CDNIVELITPRJ = '3'
                      ORDER BY CDOPERADOR, CDFILAPRIO) LOOP

       BEGIN  /*Verifica se a atividade foi incluida no cronograma ou se foi alterada as horas no cronograma manualmente*/
          SELECT IDATVTEMPALTCR, IDATIVINCLCRONO
            INTO V_IDATVTEMPALTCR, V_IDATIVINCLCRONO
            FROM PLANATIVTARE
           WHERE CDFILIAL     = P_CDFILIAL
             AND NRTAREFA     = C_TAREFAS.NRTAREFA
             AND NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE;
       EXCEPTION
          WHEN NO_DATA_FOUND THEN
             NULL;
          V_IDATVTEMPALTCR  := 'N';
          V_IDATIVINCLCRONO := 'N';
       END;

       BEGIN  /*Produtividade/Dia*/
          SELECT NVL(MAX(QTTEMPPRODIA), 0)
            INTO V_QTTEMPPRODIA
            FROM CRONOPROJETO C
           WHERE C.CDFILIAL     = P_CDFILIAL
             AND C.CDPROJETO    = P_CDPROJETO
             AND C.NRBASELINEPR = P_NRBASELINEPR
             AND C.CDOPERADOR   = P_CDOPERADOR
             AND C.NRTAREFA     = C_TAREFAS.NRTAREFA
             AND C.NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE
             AND C.CDVERCRONPRJ = (SELECT MAX(CDVERCRONPRJ)
                                     FROM CRONOPROJETO D
                                    WHERE D.CDFILIAL     = C.CDFILIAL
                                      AND D.CDPROJETO    = C.CDPROJETO
                                      AND D.CDVERCRONPRJ = NVL(P_CDVERCRONPRJ, D.CDVERCRONPRJ));
       EXCEPTION
          WHEN NO_DATA_FOUND THEN
             NULL;
          V_QTTEMPPRODIA := 0;
       END;

       /*VERIFICA SE O OPERADOR TEM PRODUTIVIDADE MEDIA INFORMADA NA OPERPROJETO*/

       BEGIN
          SELECT NVL(QTPRDIAOPPRJ,0)
            INTO V_PRODDIA
            FROM OPERPROJETO
           WHERE CDFILIAL = P_CDFILIAL
             AND CDPROJETO = P_CDPROJETO
             AND CDOPERADOR = P_CDOPERADOR;
       EXCEPTION
          WHEN NO_DATA_FOUND THEN
             NULL;
          V_PRODDIA := 0;
       END;

       IF V_PRODDIA = 0 THEN
          BEGIN
             SELECT NVL(QTPRDIAPROJ,1)
               INTO V_PRODDIA
               FROM PROJETO P
              WHERE P.CDFILIAL = P_CDFILIAL
                AND P.CDPROJETO = P_CDPROJETO;
          EXCEPTION
             WHEN NO_DATA_FOUND THEN
                NULL;
             V_PRODDIA := 1;
          END;
       END IF;

       /* Retorna a data de inicio planejada e a produtividade diria para o projeto */
       /* Tempo das atividades */

       /* Calcula a complexidade da atividade */

       V_NRCPLXITATIV := 0;

       FOR C_ATIVPROC IN (SELECT PT.NRSEQUITTARE, PR.IDREFECPLX, NVL(PR.VRFATOCPLX,0) AS VRFATOCPLX,
                                 PL.NRTAREFA, PL.NRPLATIVTARE, PL.CDPROCESSO, PL.CDFILIAL, PR.IDGERATVCONS
                            FROM PROCESSO PR, PLANATIVTARE PL, PLATVRELITTR PT
                           WHERE PR.CDPROCESSO = PL.CDPROCESSO
                             AND PL.NRTAREFA = PT.NRTAREFA
                             AND PL.NRPLATIVTARE = PT.NRPLATIVTARE
                             AND PL.NRTAREFA = C_TAREFAS.NRTAREFA
                             AND PL.NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE) LOOP

          IF C_ATIVPROC.IDREFECPLX = 'A' THEN
             BEGIN
                SELECT ROUND((I.NRCPLXIMPLEM * C_ATIVPROC.VRFATOCPLX),2)
                  INTO V_COMPLEXIDADE
                  FROM ITEMTAREFA I
                 WHERE I.CDFILIAL = C_ATIVPROC.CDFILIAL
                   AND I.NRTAREFA = C_ATIVPROC.NRTAREFA
                   AND I.NRSEQUITTARE = C_ATIVPROC.NRSEQUITTARE;
             EXCEPTION
                WHEN NO_DATA_FOUND THEN
                   NULL;
                V_COMPLEXIDADE := 0;
             END;
          ELSE
             BEGIN
                SELECT ROUND((I.NRCPLXITTARE * C_ATIVPROC.VRFATOCPLX),2)
                  INTO V_COMPLEXIDADE
                  FROM ITEMTAREFA I
                 WHERE I.CDFILIAL = C_ATIVPROC.CDFILIAL
                   AND I.NRTAREFA = C_ATIVPROC.NRTAREFA
                   AND I.NRSEQUITTARE = C_ATIVPROC.NRSEQUITTARE;
             EXCEPTION
                WHEN NO_DATA_FOUND THEN
                   NULL;
                V_COMPLEXIDADE := 0;
             END;
          END IF;

          V_NRCPLXITATIV := V_NRCPLXITATIV + V_COMPLEXIDADE;

          IF ((C_ATIVPROC.IDGERATVCONS = 'S') AND ((C_ATIVPROC.CDPROCESSO = '0000010702') OR (C_ATIVPROC.CDPROCESSO = '0000010703') OR
             (C_ATIVPROC.CDPROCESSO = '0000010708') OR (C_ATIVPROC.CDPROCESSO = '0000010709'))) THEN
             FOR C_RASTREA IN (SELECT RA.CDFILIAL, RA.NRTAREFA, RA.CDITEMTRAB, IT.NRCPLXITTRAB
                                 FROM RASTREABTARE RA, ITEMTRAB IT
                                WHERE RA.CDFILIAL   = C_ATIVPROC.CDFILIAL
                                  AND RA.NRTAREFA   = C_ATIVPROC.NRTAREFA
                                  AND RA.CDITEMTRAB = IT.CDITEMTRAB
                                  AND IDRASTREABHORIZ = 'S') LOOP
                V_NRCPLXITATIV := V_NRCPLXITATIV + (C_RASTREA.NRCPLXITTRAB * C_ATIVPROC.VRFATOCPLX);
             END LOOP;
          END IF;
       END LOOP;

       UPDATE PLANATIVTARE
          SET NRCPLXITATIV = V_NRCPLXITATIV,
              NRCPLXPLANAT = V_NRCPLXITATIV
        WHERE CDFILIAL     = P_CDFILIAL
          AND NRTAREFA     = C_TAREFAS.NRTAREFA
          AND NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE;

       /* Fim calcula a complexidade da atividade*/

       IF (V_IDATVTEMPALTCR = 'N') AND (V_IDATIVINCLCRONO = 'N') THEN
          /*ATUALIZA TEMPO DE ACORDO COM O PESO DO OPERADOR*/
          BEGIN
             SELECT ((PL.NRCPLXITATIV / PC.VRPRODMEDES) * ((100+ NVL(GC.VRPESOATIVGRC,0))/100) + NVL(QTTEMPMINATIV,0))
               INTO V_QTHRATIV
               FROM PLANATIVTARE PL, PROCESSO PC, OPERADOR OP, GRUPO_CARGO GC, CARGO CA
              WHERE PL.CDFILIAL     = P_CDFILIAL
                AND PL.NRTAREFA     = C_TAREFAS.NRTAREFA
                AND PL.NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE
                AND PL.CDPROCESSO   = PC.CDPROCESSO
                AND PL.CDOPERRESP   = OP.CDOPERADOR
                AND GC.CDGRUPOCARGO = CA.CDGRUPOCARGO
                AND OP.CDCARGO      = CA.CDCARGO;
          EXCEPTION
             WHEN NO_DATA_FOUND THEN
                NULL;
             V_QTHRATIV := 0;
          END;
       ELSE
          V_QTHRATIV := C_TAREFAS.NRDIASITCRPJ;
       END IF;

       UPDATE PLANATIVTARE
          SET QTTEMPLAITAR = V_QTHRATIV
        WHERE CDFILIAL     = P_CDFILIAL
          AND NRTAREFA     = C_TAREFAS.NRTAREFA
          AND NRPLATIVTARE = C_TAREFAS.NRPLATIVTARE;

       /*Verifica se a atividade possui precedncia e se a precedncia ja esta concluda*/
       IF (C_TAREFAS.CDPRECITCRPJ IS NOT NULL) THEN
          IF INSTR(C_TAREFAS.CDPRECITCRPJ,':') > 0 THEN
             V_PROJETO     := SUBSTR(C_TAREFAS.CDPRECITCRPJ,1,INSTR(C_TAREFAS.CDPRECITCRPJ,':')-1);
             V_PRECEDENCIA := SUBSTR(C_TAREFAS.CDPRECITCRPJ,INSTR(C_TAREFAS.CDPRECITCRPJ,':')+1,LENGTH(C_TAREFAS.CDPRECITCRPJ));

             SELECT MAX(CDVERCRONPRJ)
               INTO V_CDVERSAO
               FROM CRONOPROJETO
              WHERE CDFILIAL     = P_CDFILIAL
                AND CDPROJETO    = V_PROJETO
                AND NRBASELINEPR = P_NRBASELINEPR;
          ELSE
             V_PROJETO     := P_CDPROJETO;
             V_PRECEDENCIA := C_TAREFAS.CDPRECITCRPJ;
             V_CDVERSAO    := P_CDVERCRONPRJ;
          END IF;

          /*Verifica se a precendencia esta concluda*/
          SELECT P.IDSITATIVTAR
            INTO V_IDSITATIVTAR
            FROM CRONOPROJETO C, PLANATIVTARE P
           WHERE C.CDFILIAL     = P_CDFILIAL
             AND C.CDPROJETO    = V_PROJETO
             AND C.NRBASELINEPR = P_NRBASELINEPR
             AND C.CDVERCRONPRJ = V_CDVERSAO
             AND C.CDFILIAL     = P.CDFILIAL
             AND C.NRTAREFA     = P.NRTAREFA
             AND C.NRPLATIVTARE = P.NRPLATIVTARE
             AND INSTR('_' ||V_PRECEDENCIA||'_', '_' ||C.CDSEQCRONPRJ||'_') > 0
             AND EXISTS (SELECT MAX(DTFINITCRPRJ)
                           FROM CRONOPROJETO D
                          WHERE D.CDFILIAL     = C.CDFILIAL
                            AND D.CDPROJETO    = C.CDPROJETO
                            AND D.NRBASELINEPR = C.NRBASELINEPR
                            AND D.CDVERCRONPRJ = C.CDVERCRONPRJ
                            AND D.DTFINITCRPRJ = C.DTFINITCRPRJ
                            AND INSTR('_'||V_PRECEDENCIA||'_', '_' ||D.CDSEQCRONPRJ||'_') > 0);
       END IF;

       /*Atividades que no possuem precedncia ou que a precedncia esteja concluda
       (no dependem de outra atividade para iniciarem)*/
       IF ( (C_TAREFAS.CDPRECITCRPJ IS NULL) OR
            ((C_TAREFAS.CDPRECITCRPJ IS NOT NULL) AND ((V_IDSITATIVTAR = 'C') OR (V_IDSITATIVTAR = 'X'))) ) THEN
          /* Busca a prxima data inicial da atividade considerando apenas os dias teis*/

          IF V_QTTEMPPRODIA = 0 THEN
             BEGIN
                SELECT DTOCUPOPEPRJ, QTOCUPOPEPRJ
                  INTO V_DATAFINANT, V_DIASANT
                  FROM OCUPAOPERPROJ
                 WHERE CDFILIAL      = P_CDFILIAL
                   AND CDPROJETO     = P_CDPROJETO
                   AND CDOPERADOR    = P_CDOPERADOR
                   AND DTOCUPOPEPRJ  = (SELECT MAX(DTOCUPOPEPRJ)
                                          FROM OCUPAOPERPROJ
                                         WHERE CDFILIAL = P_CDFILIAL
                                           AND CDPROJETO = P_CDPROJETO
                                           AND CDOPERADOR = P_CDOPERADOR);
             EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  NULL;
               V_DIASANT := 0;
               V_DATAFINANT := P_DATAINIPRJ;
             END;
          ELSE
             BEGIN
                SELECT DTOCUPOPEPRJ, QTOCUPOPEPRJ
                  INTO V_DATAFINANT, V_DIASANT
                  FROM OCUPAOPERPROJ
                 WHERE CDFILIAL      = P_CDFILIAL
                   AND CDPROJETO     = P_CDPROJETO
                   AND CDOPERADOR    = P_CDOPERADOR
                   AND DTOCUPOPEPRJ = (SELECT MIN(DTOCUPOPEPRJ)
                                         FROM OCUPAOPERPROJ
                                        WHERE CDFILIAL = P_CDFILIAL
                                          AND CDPROJETO = P_CDPROJETO
                                          AND CDOPERADOR = P_CDOPERADOR
                                          AND QTOCUPOPEPRJ < V_PRODDIA);
             EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  NULL;
               V_DIASANT := 0;
               V_DATAFINANT := P_DATAINIPRJ;
             END;
          END IF;

          V_DECIMAL := 0;
          V_DIASINI := 1;
          IF V_DIASANT <> 0 THEN
             IF V_DIASANT < V_PRODDIA THEN
                V_DECIMAL := V_PRODDIA - V_DIASANT;
                V_DIASINI := 1;
             ELSE
                IF V_DIASANT = V_PRODDIA THEN
                   V_DIASANT := 0;
                   V_DIASINI := 2;
                ELSE
                   V_DIASINI := 2;
                END IF;
             END IF;
          END IF;

          SELECT MAX(DTDATA)
            INTO V_DATAINI
            FROM (SELECT ROWNUM AS NUMERO, DTDATA
                    FROM DATA
                   WHERE DTDATA >= V_DATAFINANT
                     AND NOT EXISTS (SELECT DTFERIFILI
                                       FROM FERIFILI
                                      WHERE CDFILIAL = P_CDFILIAL
                                        AND DTFERIFILI = DATA.DTDATA)
                                        AND NOT EXISTS (SELECT DTFERIOPER
                                                          FROM FERIOPER
                                                         WHERE CDOPERADOR = P_CDOPERADOR
                                                           AND DTFERIOPER = DATA.DTDATA)) D
           WHERE D.NUMERO = V_DIASINI;

          IF C_TAREFAS.QTDIAFOLGACRON > 0 THEN
             BEGIN
                V_DIASANT := 0;
                SELECT DTDATA
                  INTO V_DATAINCL
                  FROM (SELECT DTFERIFILI AS DTDATA
                          FROM FERIFILI
                         WHERE CDFILIAL = P_CDFILIAL
                           AND DTFERIFILI = V_DATAINI + C_TAREFAS.QTDIAFOLGACRON
                         UNION
                        SELECT DTFERIOPER AS DTDATA
                          FROM FERIOPER
                         WHERE DTFERIOPER = V_DATAINI + C_TAREFAS.QTDIAFOLGACRON
                           AND CDOPERADOR = P_CDOPERADOR);
                V_DATAINI :=  V_DATAINI + C_TAREFAS.QTDIAFOLGACRON;
             EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  NULL;
               V_DATAINCL := NULL;
               V_DATAINI := V_DATAINI+ C_TAREFAS.QTDIAFOLGACRON;
             END;

             WHILE (V_DATAINCL IS NOT NULL) LOOP
                V_DATAINI := V_DATAINI + 1;
                BEGIN
                   SELECT DTDATA
                     INTO V_DATAINCL
                     FROM (SELECT DTFERIFILI AS DTDATA
                             FROM FERIFILI
                            WHERE CDFILIAL = P_CDFILIAL
                              AND DTFERIFILI = V_DATAINI
                            UNION
                           SELECT DTFERIOPER AS DTDATA
                             FROM FERIOPER
                            WHERE DTFERIOPER = V_DATAINI
                              AND CDOPERADOR = P_CDOPERADOR);
                EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     NULL;
                     V_DATAINCL := NULL;
                END;
             END LOOP;

          END IF;
          /* nmero de dias que sero necessrios para a atividade com base na produtividade
             diria do projeto */

          IF V_QTTEMPPRODIA = 0 THEN /*Produtividade/Dia*/
             V_DIAS := CEIL((V_QTHRATIV + V_DIASANT)/V_PRODDIA);
          ELSE
             V_DIAS := CEIL(V_QTHRATIV /V_QTTEMPPRODIA);
          END IF;

          IF (V_DIAS = 0) THEN
             V_DIAS := 1;
          END IF;

          /* Busca a data final da atividade considerando apenas os dias teis*/
          SELECT MAX(DTDATA)
            INTO V_DATAFINAL
            FROM (SELECT ROWNUM AS NUMERO, DTDATA
                    FROM DATA
                   WHERE DTDATA >= V_DATAINI
                     AND NOT EXISTS (SELECT DTFERIFILI
                                       FROM FERIFILI
                                      WHERE CDFILIAL = P_CDFILIAL
                                        AND DTFERIFILI = DATA.DTDATA)
                     AND NOT EXISTS (SELECT DTFERIOPER
                                       FROM FERIOPER
                                      WHERE CDOPERADOR = P_CDOPERADOR
                                        AND DTFERIOPER = DATA.DTDATA)) D
           WHERE D.NUMERO = V_DIAS;

          UPDATE CRONOPROJETO
             SET NRDIASITCRPJ = V_QTHRATIV,
	         QTHRANTPATIVTAR = V_QTHRATIV,
                 DTINIITCRPRJ = V_DATAINI,
                 DTFINITCRPRJ = V_DATAFINAL
           WHERE CDFILIAL     = P_CDFILIAL
             AND CDPROJETO    = P_CDPROJETO
             AND NRBASELINEPR = P_NRBASELINEPR
             AND CDVERCRONPRJ = P_CDVERCRONPRJ
             AND CDSEQCRONPRJ = C_TAREFAS.CDSEQCRONPRJ;

          /*INSERE NA TABELA OCUPACAO DO OPERADOR POR PROJETO*/
          V_QTHRPROD := V_QTHRATIV;
          WHILE (V_QTHRPROD > 0) LOOP
             BEGIN
                SELECT DTDATA
                  INTO V_DATAATIV
                  FROM (SELECT DTFERIFILI AS DTDATA
                          FROM FERIFILI
                         WHERE CDFILIAL = P_CDFILIAL
                           AND DTFERIFILI = V_DATAINI
                         UNION
                        SELECT DTFERIOPER AS DTDATA
                          FROM FERIOPER
                         WHERE DTFERIOPER = V_DATAINI
                           AND CDOPERADOR = P_CDOPERADOR);
             EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  NULL;
               V_DATAATIV := NULL;
             END;

             WHILE (V_DATAATIV IS NOT NULL) LOOP
                V_DATAINI := V_DATAINI + 1;
                BEGIN
                   SELECT DTDATA
                     INTO V_DATAATIV
                     FROM (SELECT DTFERIFILI AS DTDATA
                             FROM FERIFILI
                            WHERE CDFILIAL = P_CDFILIAL
                              AND DTFERIFILI = V_DATAINI
                            UNION
                           SELECT DTFERIOPER AS DTDATA
                             FROM FERIOPER
                            WHERE DTFERIOPER = V_DATAINI
                              AND CDOPERADOR = P_CDOPERADOR);
                EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     NULL;
                     V_DATAATIV := NULL;
                END;
             END LOOP;

             BEGIN
                SELECT DTOCUPOPEPRJ, QTOCUPOPEPRJ
                  INTO V_DTOCUPOPER, V_QTOPUCOPER
                  FROM OCUPAOPERPROJ
                 WHERE CDFILIAL      = P_CDFILIAL
                   AND CDPROJETO     = P_CDPROJETO
                   AND CDOPERADOR    = P_CDOPERADOR
                   AND DTOCUPOPEPRJ = V_DATAINI;
             EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  NULL;
               V_QTOPUCOPER := 0;
               V_DTOCUPOPER := NULL;
             END;

             /*SE NAO EXISTIR DATA NA TABELA */
             IF V_DTOCUPOPER IS NULL THEN
                IF V_QTTEMPPRODIA = 0 THEN /*Produtividade/Dia*/
                   IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER) THEN
                      INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                         (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTHRPROD);
                      V_QTHRPROD := 0;
                   ELSE
                      INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                         (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_PRODDIA);
                      V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                   END IF;
                ELSE
                   IF V_QTHRPROD <= V_QTTEMPPRODIA  THEN
                      INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                         (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTHRPROD);
                      V_QTHRPROD := 0;
                   ELSE
                      INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                         (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTTEMPPRODIA);
                      V_QTHRPROD := V_QTHRPROD - V_QTTEMPPRODIA;
                   END IF;
                END IF;
             ELSE
                IF V_QTTEMPPRODIA = 0 THEN
                   IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER)THEN
                      UPDATE OCUPAOPERPROJ
                         SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTHRPROD
                       WHERE CDFILIAL      = P_CDFILIAL
                         AND CDPROJETO     = P_CDPROJETO
                         AND CDOPERADOR    = P_CDOPERADOR
                         AND DTOCUPOPEPRJ = V_DATAINI;
                      V_QTHRPROD := 0;
                   ELSE
                      UPDATE OCUPAOPERPROJ
                         SET QTOCUPOPEPRJ = V_PRODDIA
                       WHERE CDFILIAL      = P_CDFILIAL
                         AND CDPROJETO     = P_CDPROJETO
                         AND CDOPERADOR    = P_CDOPERADOR
                         AND DTOCUPOPEPRJ = V_DATAINI;

                      V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                   END IF;
                ELSE
                   IF V_QTHRPROD <= V_QTTEMPPRODIA THEN
                      IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER)THEN
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTHRPROD
                          WHERE CDFILIAL      = P_CDFILIAL
                            AND CDPROJETO     = P_CDPROJETO
                            AND CDOPERADOR    = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;
                         V_QTHRPROD := 0;
                      ELSE
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = V_PRODDIA
                          WHERE CDFILIAL      = P_CDFILIAL
                            AND CDPROJETO     = P_CDPROJETO
                            AND CDOPERADOR    = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;

                         V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                      END IF;
                   ELSE
                      IF V_QTTEMPPRODIA <= (V_PRODDIA - V_QTOPUCOPER)THEN
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTTEMPPRODIA
                          WHERE CDFILIAL      = P_CDFILIAL
                            AND CDPROJETO     = P_CDPROJETO
                            AND CDOPERADOR    = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;
                         V_QTHRPROD := V_QTHRPROD - V_QTTEMPPRODIA;
                      ELSE
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = V_PRODDIA
                          WHERE CDFILIAL     = P_CDFILIAL
                            AND CDPROJETO    = P_CDPROJETO
                            AND CDOPERADOR   = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;

                         V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                      END IF;
                   END IF;
                END IF;
             END IF;
             V_DECIMAL := 0;
             V_DATAINI := V_DATAINI + 1;
          END LOOP;


          BEGIN
             SELECT DTINIITCRPRJ, DTFINITCRPRJ
               INTO  V_DATAINI, V_DATAFINAL
               FROM CRONOPROJETO
              WHERE CDFILIAL     = P_CDFILIAL
                AND CDPROJETO    = P_CDPROJETO
                AND NRBASELINEPR = P_NRBASELINEPR
                AND CDVERCRONPRJ = P_CDVERCRONPRJ
                AND CDSEQCRONPRJ = C_TAREFAS.CDSEQCRONPRJ;
          EXCEPTION
             WHEN NO_DATA_FOUND THEN
                NULL;
          END;
       ELSE
          /*Atividades que possuem precedncia(dependem de outra atividades para iniciarem)*/
          /* Retorna a maior data das atividades que precedem a atividade que est sendo calculada */
          IF INSTR(C_TAREFAS.CDPRECITCRPJ,':') > 0 THEN
             V_PROJETO     := SUBSTR(C_TAREFAS.CDPRECITCRPJ,1,INSTR(C_TAREFAS.CDPRECITCRPJ,':')-1);
             V_PRECEDENCIA := SUBSTR(C_TAREFAS.CDPRECITCRPJ,INSTR(C_TAREFAS.CDPRECITCRPJ,':')+1,LENGTH(C_TAREFAS.CDPRECITCRPJ));

             SELECT MAX(CDVERCRONPRJ)
               INTO V_CDVERSAO
               FROM CRONOPROJETO
              WHERE CDFILIAL     = P_CDFILIAL
                AND CDPROJETO    = V_PROJETO
                AND NRBASELINEPR = P_NRBASELINEPR;
          ELSE
             V_PROJETO     := P_CDPROJETO;
             V_PRECEDENCIA := C_TAREFAS.CDPRECITCRPJ;
             V_CDVERSAO    := P_CDVERCRONPRJ;
          END IF;

          SELECT MAX(DTFINITCRPRJ), MAX(CDOPERADOR), MAX(CDFILAPRIO), MAX(NVL(NRDIASITCRPJ,0))
            INTO V_DATAFINANT, V_OPERADORPREC, V_FILA, V_DIASANT
            FROM CRONOPROJETO C
           WHERE CDFILIAL     = P_CDFILIAL
             AND CDPROJETO    = V_PROJETO
             AND NRBASELINEPR = P_NRBASELINEPR
             AND CDVERCRONPRJ = V_CDVERSAO
             AND INSTR('_' ||V_PRECEDENCIA||'_', '_' ||CDSEQCRONPRJ||'_') > 0
             AND EXISTS (SELECT MAX(DTFINITCRPRJ)
                           FROM CRONOPROJETO D
                          WHERE CDFILIAL       = C.CDFILIAL
                            AND CDPROJETO      = C.CDPROJETO
                            AND NRBASELINEPR   = C.NRBASELINEPR
                            AND CDVERCRONPRJ   = C.CDVERCRONPRJ
                            AND C.DTFINITCRPRJ = D.DTFINITCRPRJ
                            AND INSTR('_'||V_PRECEDENCIA||'_', '_' ||D.CDSEQCRONPRJ||'_') > 0);


          BEGIN
             SELECT QTOCUPOPEPRJ
               INTO V_DIASANT
               FROM OCUPAOPERPROJ
              WHERE CDFILIAL     = P_CDFILIAL
                AND CDPROJETO    = P_CDPROJETO
                AND CDOPERADOR   = P_CDOPERADOR
                AND DTOCUPOPEPRJ = V_DATAFINANT;
          EXCEPTION
            WHEN NO_DATA_FOUND THEN
               NULL;
            V_DIASANT := 0;
          END;

          /* A atividade antecessora j possui a data final preenchida */
          IF V_DATAFINANT IS NOT NULL THEN
             /* Calcula quantas horas so necessrias para realizar a parte decimal do dia e se ultrapassa
                a produtividade mdia, pois se ultrapassar a data inicial da prxima atividade dever
                no prximo dia*/
             V_DECIMAL := 0;
             V_DIASINI := 1;
             IF V_DIASANT <> 0 THEN
                IF V_DIASANT < V_PRODDIA THEN
                   V_DECIMAL := V_PRODDIA - V_DIASANT;
                   V_DIASINI := 1;
                ELSE
                   V_DIASINI := 2;
                END IF;
             END IF;

             /* Busca a prxima data inicial da atividade considerando apenas os dias teis*/
             SELECT MAX(DTDATA)
               INTO V_DATAINI
               FROM (SELECT ROWNUM AS NUMERO, DTDATA
                       FROM DATA
                      WHERE DTDATA >= V_DATAFINANT
                        AND NOT EXISTS (SELECT DTFERIFILI
                                          FROM FERIFILI
                                         WHERE CDFILIAL = P_CDFILIAL
                                           AND DTFERIFILI = DATA.DTDATA)
                        AND NOT EXISTS (SELECT DTFERIOPER
                                          FROM FERIOPER
                                         WHERE CDOPERADOR = P_CDOPERADOR
                                           AND DTFERIOPER = DATA.DTDATA)) D
              WHERE D.NUMERO = 1;

             IF C_TAREFAS.QTDIAFOLGACRON > 0 THEN

                BEGIN
                   SELECT DTDATA
                     INTO V_DATAINCL
                     FROM (SELECT DTFERIFILI AS DTDATA
                             FROM FERIFILI
                            WHERE CDFILIAL = P_CDFILIAL
                              AND DTFERIFILI = V_DATAINI + C_TAREFAS.QTDIAFOLGACRON
                            UNION
                           SELECT DTFERIOPER AS DTDATA
                             FROM FERIOPER
                            WHERE DTFERIOPER = V_DATAINI + C_TAREFAS.QTDIAFOLGACRON
                              AND CDOPERADOR = P_CDOPERADOR);
                   V_DATAINI :=  V_DATAINI + C_TAREFAS.QTDIAFOLGACRON;
                EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     NULL;
                    V_DATAINCL := NULL;
                    V_DATAINI := V_DATAINI+ C_TAREFAS.QTDIAFOLGACRON;
                END;


                WHILE (V_DATAINCL IS NOT NULL) LOOP
                   V_DATAINI := V_DATAINI + 1;
                   BEGIN
                      SELECT DTDATA
                        INTO V_DATAINCL
                        FROM (SELECT DTFERIFILI AS DTDATA
                                FROM FERIFILI
                               WHERE CDFILIAL = P_CDFILIAL
                                 AND DTFERIFILI = V_DATAINI
                               UNION
                              SELECT DTFERIOPER AS DTDATA
                                FROM FERIOPER
                               WHERE DTFERIOPER = V_DATAINI
                                 AND CDOPERADOR = P_CDOPERADOR);
                   EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        NULL;
                        V_DATAINCL := NULL;
                   END;
                END LOOP;
             END IF;


             /* nmero de dias que sero necessrios para a atividade com base na produtividade
                diria do projeto */
             IF V_QTTEMPPRODIA = 0 THEN
                V_DIAS := CEIL((V_QTHRATIV + V_DIASANT)/V_PRODDIA);
             ELSE
                V_DIAS := CEIL(V_QTHRATIV /V_QTTEMPPRODIA);
             END IF;

             IF (V_DIAS = 0) THEN
                V_DIAS := 1;
             END IF;

             /* Busca a data final da atividade considerando apenas os dias teis*/
             SELECT MAX(DTDATA)
               INTO V_DATAFINAL
               FROM (SELECT ROWNUM AS NUMERO, DTDATA
                       FROM DATA
                      WHERE DTDATA >= V_DATAINI
                        AND NOT EXISTS (SELECT DTFERIFILI
                                          FROM FERIFILI
                                         WHERE CDFILIAL = P_CDFILIAL
                                           AND DTFERIFILI = DATA.DTDATA)
                        AND NOT EXISTS (SELECT DTFERIOPER
                                          FROM FERIOPER
                                         WHERE CDOPERADOR = P_CDOPERADOR
                                           AND DTFERIOPER = DATA.DTDATA)) D
             WHERE D.NUMERO = V_DIAS;

             UPDATE CRONOPROJETO
                SET NRDIASITCRPJ = V_QTHRATIV,
		    QTHRANTPATIVTAR = V_QTHRATIV,
                    DTINIITCRPRJ = V_DATAINI,
                    DTFINITCRPRJ = V_DATAFINAL
              WHERE CDFILIAL     = P_CDFILIAL
                AND CDPROJETO    = P_CDPROJETO
                AND NRBASELINEPR = P_NRBASELINEPR
                AND CDVERCRONPRJ = P_CDVERCRONPRJ
                AND CDSEQCRONPRJ = C_TAREFAS.CDSEQCRONPRJ ;

             V_QTHRPROD := V_QTHRATIV;
             WHILE (V_QTHRPROD > 0) LOOP
                BEGIN
                   SELECT DTDATA
                     INTO V_DATAATIV
                     FROM (SELECT DTFERIFILI AS DTDATA
                             FROM FERIFILI
                            WHERE CDFILIAL = P_CDFILIAL
                              AND DTFERIFILI = V_DATAINI
                            UNION
                           SELECT DTFERIOPER AS DTDATA
                             FROM FERIOPER
                            WHERE DTFERIOPER = V_DATAINI
                              AND CDOPERADOR = P_CDOPERADOR);
                EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     NULL;
                  V_DATAATIV := NULL;
                END;

                WHILE (V_DATAATIV IS NOT NULL) LOOP
                   V_DATAINI := V_DATAINI + 1;
                   BEGIN
                      SELECT DTDATA
                        INTO V_DATAATIV
                        FROM (SELECT DTFERIFILI AS DTDATA
                                FROM FERIFILI
                               WHERE CDFILIAL = P_CDFILIAL
                                 AND DTFERIFILI = V_DATAINI
                               UNION
                              SELECT DTFERIOPER AS DTDATA
                                FROM FERIOPER
                               WHERE DTFERIOPER = V_DATAINI
                                 AND CDOPERADOR = P_CDOPERADOR);
                   EXCEPTION
                     WHEN NO_DATA_FOUND THEN
                        NULL;
                        V_DATAATIV := NULL;
                   END;
                END LOOP;

                BEGIN
                   SELECT DTOCUPOPEPRJ, QTOCUPOPEPRJ
                     INTO V_DTOCUPOPER, V_QTOPUCOPER
                     FROM OCUPAOPERPROJ
                    WHERE CDFILIAL      = P_CDFILIAL
                      AND CDPROJETO     = P_CDPROJETO
                      AND CDOPERADOR    = P_CDOPERADOR
                      AND DTOCUPOPEPRJ = V_DATAINI;
                EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     NULL;
                  V_QTOPUCOPER := 0;
                  V_DTOCUPOPER := NULL;
                END;

                /*SE NAO EXISTIR DATA NA TABELA */
                IF V_DTOCUPOPER IS NULL THEN
                   IF V_QTTEMPPRODIA = 0 THEN /*Produtividade/Dia*/
                      IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER) THEN
                         INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                            (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTHRPROD);
                         V_QTHRPROD := 0;
                      ELSE
                         INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                            (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_PRODDIA);
                         V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                      END IF;
                   ELSE
                      IF V_QTHRPROD <= (V_QTTEMPPRODIA - V_QTOPUCOPER) THEN
                         INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                            (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTHRPROD);
                         V_QTHRPROD := 0;
                      ELSE
                         INSERT INTO OCUPAOPERPROJ (CDFILIAL, CDPROJETO, CDOPERADOR,DTOCUPOPEPRJ, QTOCUPOPEPRJ) VALUES
                            (P_CDFILIAL, P_CDPROJETO, P_CDOPERADOR, V_DATAINI, V_QTTEMPPRODIA);
                         V_QTHRPROD := V_QTHRPROD - (V_QTTEMPPRODIA - V_QTOPUCOPER);
                      END IF;
                   END IF;
                ELSE
                   IF V_QTTEMPPRODIA = 0 THEN
                      IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER)THEN
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTHRPROD
                          WHERE CDFILIAL      = P_CDFILIAL
                            AND CDPROJETO     = P_CDPROJETO
                            AND CDOPERADOR    = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;
                         V_QTHRPROD := 0;
                      ELSE
                         UPDATE OCUPAOPERPROJ
                            SET QTOCUPOPEPRJ = V_PRODDIA
                          WHERE CDFILIAL      = P_CDFILIAL
                            AND CDPROJETO     = P_CDPROJETO
                            AND CDOPERADOR    = P_CDOPERADOR
                            AND DTOCUPOPEPRJ = V_DATAINI;

                         V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                      END IF;
                   ELSE
                      IF V_QTHRPROD <= V_QTTEMPPRODIA THEN
                         IF V_QTHRPROD <= (V_PRODDIA - V_QTOPUCOPER)THEN
                            UPDATE OCUPAOPERPROJ
                               SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTHRPROD
                             WHERE CDFILIAL      = P_CDFILIAL
                               AND CDPROJETO     = P_CDPROJETO
                               AND CDOPERADOR    = P_CDOPERADOR
                               AND DTOCUPOPEPRJ = V_DATAINI;
                            V_QTHRPROD := 0;
                         ELSE
                            UPDATE OCUPAOPERPROJ
                               SET QTOCUPOPEPRJ = V_PRODDIA
                             WHERE CDFILIAL      = P_CDFILIAL
                               AND CDPROJETO     = P_CDPROJETO
                               AND CDOPERADOR    = P_CDOPERADOR
                               AND DTOCUPOPEPRJ = V_DATAINI;

                            V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                         END IF;
                      ELSE
                         IF V_QTTEMPPRODIA <= (V_PRODDIA - V_QTOPUCOPER) THEN
                            UPDATE OCUPAOPERPROJ
                               SET QTOCUPOPEPRJ = QTOCUPOPEPRJ + V_QTTEMPPRODIA
                             WHERE CDFILIAL      = P_CDFILIAL
                               AND CDPROJETO     = P_CDPROJETO
                               AND CDOPERADOR    = P_CDOPERADOR
                               AND DTOCUPOPEPRJ = V_DATAINI;
                            V_QTHRPROD := V_QTHRPROD - V_QTTEMPPRODIA;
                         ELSE
                            UPDATE OCUPAOPERPROJ
                               SET QTOCUPOPEPRJ = V_PRODDIA
                             WHERE CDFILIAL     = P_CDFILIAL
                               AND CDPROJETO    = P_CDPROJETO
                               AND CDOPERADOR   = P_CDOPERADOR
                               AND DTOCUPOPEPRJ = V_DATAINI;

                            V_QTHRPROD := V_QTHRPROD - (V_PRODDIA - V_QTOPUCOPER);
                         END IF;
                      END IF;
                   END IF;
                END IF;
                V_DECIMAL := 0;
                V_DATAINI := V_DATAINI + 1;
             END LOOP;
          ELSE
             /* A data final da atividade antecessora no est preenchida*/
             CALCULA_DATAS(P_CDFILIAL, P_CDPROJETO, P_NRBASELINEPR, P_CDVERCRONPRJ, V_OPERADORPREC, V_FILA,P_DATAINIPRJ);
             /* Volta para calcular a data da atividade que necessitou de outra atividade */
             CALCULA_DATAS(P_CDFILIAL, P_CDPROJETO, P_NRBASELINEPR, P_CDVERCRONPRJ, P_CDOPERADOR, C_TAREFAS.CDFILAPRIO,P_DATAINIPRJ);
          END IF;
       END IF;
   END LOOP;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('CALCULA_DATAS','5.05.015'); END;
/

